﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using ScheduleCRUDJS.Models;

namespace ScheduleCRUDJS.Models
{
    public static class AppointmentRepository
    {
       

        public static Appointment Insert(Appointment app)
        {
            ScheduleDataDataContext db = new ScheduleDataDataContext();
            DateTime startTime = Convert.ToDateTime(app.StartTime);
            DateTime endTime = Convert.ToDateTime(app.EndTime);
            Appointment appoint = new Appointment()
            {
                Id = app.Id,
                StartTime = startTime,
                EndTime = endTime,
                Subject = app.Subject,
                Recurrence = app.Recurrence,
                AllDay = app.AllDay,
                Owner=app.Owner,
                RecurrenceRule = app.RecurrenceRule,
            };
            db.Appointments.InsertOnSubmit(appoint);
            db.SubmitChanges();
            return appoint;
        }

        

        public static Appointment Remove(int appId)
        {
            ScheduleDataDataContext db = new ScheduleDataDataContext();
            Appointment removeApp = db.Appointments.Where(c => c.Id == appId).FirstOrDefault();
            if (removeApp != null)
                db.Appointments.DeleteOnSubmit(removeApp);
            db.SubmitChanges();
            return removeApp;
        }

        public static Appointment Update(Appointment app)
        {
            ScheduleDataDataContext db = new ScheduleDataDataContext();
            Appointment appoint = db.Appointments.Single(A => A.Id == Convert.ToInt32(app.Id));
            var filterData = db.Appointments.Where(c => c.Id == Convert.ToInt32(app.Id));
            if (filterData.Count() > 0)
            {
                DateTime startTime = Convert.ToDateTime(app.StartTime);
                DateTime endTime = Convert.ToDateTime(app.EndTime);
                appoint.StartTime = startTime;
                appoint.EndTime = endTime;
                appoint.Subject = app.Subject;
                appoint.Recurrence = app.Recurrence;
                appoint.AllDay = app.AllDay;
                appoint.RecurrenceRule = app.RecurrenceRule;
            }
            db.SubmitChanges();
            return appoint;
            
        }

        public static List<Appointment> FilterAppointment(DateTime CurrentDate, String CurrentAction, String CurrentView, String value) // here we are filtering the appointment with the extra parameter "value"
        {
            ScheduleDataDataContext db = new ScheduleDataDataContext();
            DateTime CurrDate = Convert.ToDateTime(CurrentDate);
            DateTime StartDate = FirstWeekDate(CurrDate.Date);
            DateTime EndDate = FirstWeekDate(CurrDate.Date);
            List<Appointment> appointmentList = db.Appointments.ToList();
            
            switch (CurrentView)
            {
                case "day":
                    StartDate = CurrentDate;
                    EndDate = CurrentDate;
                    break;
                case "week":
                    EndDate = EndDate.AddDays(7);
                    break;
                case "workweek":
                    EndDate = EndDate.AddDays(5);
                    break;
                case "month":
                    StartDate = CurrDate.Date.AddDays(-CurrDate.Day + 1);
                    EndDate = StartDate.AddMonths(1);
                    break;
            }
            appointmentList = db.Appointments.ToList().Where(app => app.StartTime >= StartDate && app.StartTime <= EndDate && app.Owner==value || app.Recurrence == 1).ToList();// here particular date DefaultSchedule is filtered          
            return appointmentList;
        }

        internal static DateTime FirstWeekDate(DateTime CurrentDate)
        {
            try
            {
                DateTime FirstDayOfWeek = CurrentDate;
                DayOfWeek WeekDay = FirstDayOfWeek.DayOfWeek;
                switch (WeekDay)
                {
                    case DayOfWeek.Sunday:
                        break;
                    case DayOfWeek.Monday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-1);
                        break;
                    case DayOfWeek.Tuesday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-2);
                        break;
                    case DayOfWeek.Wednesday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-3);
                        break;
                    case DayOfWeek.Thursday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-4);
                        break;
                    case DayOfWeek.Friday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-5);
                        break;
                    case DayOfWeek.Saturday:
                        FirstDayOfWeek = FirstDayOfWeek.AddDays(-6);
                        break;
                }
                return (FirstDayOfWeek);
            }
            catch
            {
                return DateTime.Now;
            }
        }
    }
}